@@ -105,7 +105,7 @@ const char *SkywatcherAPI::mountTypeToString(uint8_t type)
         case AZGTI:
             return "AZ-GTi";
         default:
-            return "Uknown";
+            return "Unknown";
     }
 }
 
@@ -206,10 +206,10 @@ bool SkywatcherAPI::CheckIfDCMotor()
     return false;
 }
 
-bool SkywatcherAPI::IsMerlinMount() const
-{
-    return MountCode >= 0x80 && MountCode < 0x90;
-}
+// bool SkywatcherAPI::IsMerlinMount() const
+// {
+//     return MountCode >= 0x80 && MountCode < 0x90;
+// }
 
 long SkywatcherAPI::DegreesPerSecondToClocksTicksPerMicrostep(AXISID Axis, double DegreesPerSecond)
 {
@@ -277,8 +277,8 @@ bool SkywatcherAPI::GetMicrostepsPerRevolution(AXISID Axis)
     if (MountCode == _114GT)
         tmpMicrostepsPerRevolution = 0x205318; // for 114GT mount
 
-    if (IsMerlinMount())
-        tmpMicrostepsPerRevolution = (long)((double)tmpMicrostepsPerRevolution * 0.655);
+    // if (IsMerlinMount())
+    //     tmpMicrostepsPerRevolution = (long)((double)tmpMicrostepsPerRevolution * 0.655);
 
     MicrostepsPerRevolution[(int)Axis] = tmpMicrostepsPerRevolution;
 
@@ -915,7 +915,7 @@ bool SkywatcherAPI::TalkWithAxis(AXISID Axis, SkywatcherCommand Command, std::st
         {
             memset(response, '\0', SKYWATCHER_MAX_CMD);
             // If we get less than 2 bytes then it must be an error (=\r is a valid response).
-            if ( (errorCode = tty_read_section(MyPortFD, response, 0x0D, SKYWATCHER_TIMEOUT, &bytesRead)) != TTY_OK
+            if ( (errorCode = tty_read_section_expanded(MyPortFD, response, 0x0D, SKYWATCHER_TIMEOUT_S, SKYWATCHER_TIMEOUT_US, &bytesRead)) != TTY_OK
                     || bytesRead < 2)
             {
                 if (retries == SKYWATCHER_MAX_RETRTY - 1)
