@@ -4,10 +4,19 @@
 #include "base64.h"
 #include "userio.h"
 #include "indiuserio.h"
+#include "indiutility.h"
 
 #include <string.h>
 #include <stdlib.h>
 #include <assert.h>
+#include <sys/stat.h>
+
+#ifdef _WIN32
+#include <direct.h>
+#include <Windows.h>
+#endif
+
+#define MAXRBUF 2048
 
 /** \section IUSave */
 
@@ -35,7 +44,9 @@ void IUSaveConfigBLOB(FILE *fp, const IBLOBVectorProperty *bvp)
 void IUSaveText(IText *tp, const char *newtext)
 {
     /* copy in fresh string */
-    tp->text = strcpy(realloc(tp->text, strlen(newtext) + 1), newtext);
+    size_t size = strlen(newtext) + 1;
+    tp->text = realloc(tp->text, size);
+    memcpy(tp->text, newtext, size);
 }
 
 int IUSaveBLOB(IBLOB *bp, int size, int blobsize, char *blob, char *format)
@@ -43,10 +54,209 @@ int IUSaveBLOB(IBLOB *bp, int size, int blobsize, char *blob, char *format)
     bp->bloblen = blobsize;
     bp->size    = size;
     bp->blob    = blob;
-    strncpy(bp->format, format, MAXINDIFORMAT);
+    indi_strlcpy(bp->format, format, MAXINDIFORMAT);
     return 0;
 }
 
+/** Get configuration root XML pointer
+ *  N.B. Must be freed by the caller */
+XMLEle *configRootFP(const char *device)
+{
+    char configFileName[MAXRBUF];
+    char configDir[MAXRBUF];
+    char errmsg[MAXRBUF];
+    struct stat st;
+    FILE *fp = NULL;
+
+    snprintf(configDir, MAXRBUF-1, "%s/.indi/", getenv("HOME"));
+
+        if (getenv("INDICONFIG"))
+            strncpy(configFileName, getenv("INDICONFIG"), MAXRBUF-1);
+        else
+            snprintf(configFileName, MAXRBUF-1, "%s%s_config.xml", configDir, device);
+
+
+    if (stat(configDir, &st) != 0)
+    {
+#ifdef _WIN32
+        if (mkdir(configDir) != 0)
+#else
+        if (mkdir(configDir, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) < 0)
+#endif
+            return NULL;
+    }
+
+    stat(configFileName, &st);
+    
+#ifdef _WIN32
+    BOOL isAdmin = FALSE;
+    SID_IDENTIFIER_AUTHORITY NtAuthority = SECURITY_NT_AUTHORITY;
+    PSID AdministratorsGroup;
+    if (AllocateAndInitializeSid(&NtAuthority, 2, SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS, 0, 0, 0, 0, 0, 0, &AdministratorsGroup))
+    {
+        if (CheckTokenMembership(NULL, AdministratorsGroup, &isAdmin) == FALSE)
+            isAdmin = FALSE;
+        FreeSid(AdministratorsGroup);
+    }
+    if ((st.st_uid == 0 && st.st_mode & S_IFDIR) || (st.st_gid == 0 && isAdmin == TRUE))
+        return NULL;
+#else
+    /* If file is owned by root and current user is NOT root then abort */
+    if ( (st.st_uid == 0 && getuid() != 0) || (st.st_gid == 0 && getgid() != 0) )
+        return NULL;
+#endif
+
+    fp = fopen(configFileName, "r");
+    if (fp == NULL)
+        return NULL;
+
+    LilXML *lp = newLilXML();
+
+    XMLEle *root = readXMLFile(fp, lp, errmsg);
+
+    delLilXML(lp);
+    fclose(fp);
+
+    return root;
+}
+/** \section IULoad */
+
+int IULoadConfigNumber(const INumberVectorProperty *nvp)
+{
+    char errmsg[1024];
+    char *rdev, *rname;
+    int foundCounter = 0;
+    XMLEle *root = configRootFP(nvp->device);
+    if (root == NULL)
+        return -1;
+
+    XMLEle *ep = NULL;
+
+    for (ep = nextXMLEle(root, 1); ep != NULL; ep = nextXMLEle(root, 0))
+    {
+        /* pull out device and name */
+        if (crackDN(ep, &rdev, &rname, errmsg) < 0)
+        {
+            delXMLEle(root);
+            return -1;
+        }
+
+        // It doesn't belong to our device??
+        if (strcmp(nvp->device, rdev))
+            continue;
+
+        if (!strcmp(nvp->name, rname))
+        {
+            XMLEle *element = NULL;
+            for (element = nextXMLEle(ep, 1); element != NULL; element = nextXMLEle(ep, 0))
+            {
+                INumber *member = IUFindNumber(nvp, findXMLAttValu(element, "name"));
+                if (member)
+                {
+                    member->value = atof(pcdataXMLEle(element));
+                    foundCounter++;
+                }
+            }
+            break;
+        }
+    }
+
+    delXMLEle(root);
+    return foundCounter;
+}
+
+int IULoadConfigText(const ITextVectorProperty *tvp)
+{
+    char errmsg[1024];
+    char *rdev, *rname;
+    int foundCounter = 0;
+    XMLEle *root = configRootFP(tvp->device);
+    if (root == NULL)
+        return -1;
+
+    XMLEle *ep = NULL;
+
+    for (ep = nextXMLEle(root, 1); ep != NULL; ep = nextXMLEle(root, 0))
+    {
+        /* pull out device and name */
+        if (crackDN(ep, &rdev, &rname, errmsg) < 0)
+        {
+            delXMLEle(root);
+            return -1;
+        }
+
+        // It doesn't belong to our device??
+        if (strcmp(tvp->device, rdev))
+            continue;
+
+        if (!strcmp(tvp->name, rname))
+        {
+            XMLEle *element = NULL;
+            for (element = nextXMLEle(ep, 1); element != NULL; element = nextXMLEle(ep, 0))
+            {
+                IText *member = IUFindText(tvp, findXMLAttValu(element, "name"));
+                if (member)
+                {
+                    IUSaveText(member, pcdataXMLEle(element));
+                    foundCounter++;
+                }
+            }
+            break;
+        }
+    }
+
+    delXMLEle(root);
+    return foundCounter;
+}
+
+int IULoadConfigSwitch(const ISwitchVectorProperty *svp)
+{
+    char errmsg[1024];
+    char *rdev, *rname;
+    int foundCounter = 0;
+    XMLEle *root = configRootFP(svp->device);
+    if (root == NULL)
+        return -1;
+
+    XMLEle *ep = NULL;
+
+    for (ep = nextXMLEle(root, 1); ep != NULL; ep = nextXMLEle(root, 0))
+    {
+        /* pull out device and name */
+        if (crackDN(ep, &rdev, &rname, errmsg) < 0)
+        {
+            delXMLEle(root);
+            return -1;
+        }
+
+        // It doesn't belong to our device??
+        if (strcmp(svp->device, rdev))
+            continue;
+
+        if (!strcmp(svp->name, rname))
+        {
+            XMLEle *element = NULL;
+            for (element = nextXMLEle(ep, 1); element != NULL; element = nextXMLEle(ep, 0))
+            {
+                ISwitch *member = IUFindSwitch(svp, findXMLAttValu(element, "name"));
+                if (member)
+                {
+                    ISState state;
+                    if (crackISState(pcdataXMLEle(element), &state) == 0)
+                    {
+                        member->s = state;
+                        foundCounter++;
+                    }
+                }
+            }
+            break;
+        }
+    }
+
+    delXMLEle(root);
+    return foundCounter;
+}
+
 /** \section IUFind */
 
 /* find a member of an IText vector, else NULL */
@@ -154,12 +364,9 @@ void IUResetSwitch(ISwitchVectorProperty *svp)
 
 void IUFillSwitch(ISwitch *sp, const char *name, const char *label, ISState s)
 {
-    strncpy(sp->name, name, sizeof(sp->name));
+    indi_strlcpy(sp->name, name, sizeof(sp->name));
 
-    if (label[0])
-        strncpy(sp->label, label, sizeof(sp->label));
-    else
-        strncpy(sp->label, name, sizeof(sp->label));
+    indi_strlcpy(sp->label, label[0] ? label : name, sizeof(sp->label));
 
     sp->s   = s;
     sp->svp = NULL;
@@ -168,12 +375,9 @@ void IUFillSwitch(ISwitch *sp, const char *name, const char *label, ISState s)
 
 void IUFillLight(ILight *lp, const char *name, const char *label, IPState s)
 {
-    strncpy(lp->name, name, sizeof(lp->name));
+    indi_strlcpy(lp->name, name, sizeof(lp->name));
 
-    if (label[0])
-        strncpy(lp->label, label, sizeof(lp->label));
-    else
-        strncpy(lp->label, name, sizeof(lp->label));
+    indi_strlcpy(lp->label, label[0] ? label : name, sizeof(lp->label));
 
     lp->s   = s;
     lp->lvp = NULL;
@@ -183,14 +387,11 @@ void IUFillLight(ILight *lp, const char *name, const char *label, IPState s)
 void IUFillNumber(INumber *np, const char *name, const char *label, const char *format, double min, double max,
                   double step, double value)
 {
-    strncpy(np->name, name, sizeof(np->name));
+    indi_strlcpy(np->name, name, sizeof(np->name));
 
-    if (label[0])
-        strncpy(np->label, label, sizeof(np->label));
-    else
-        strncpy(np->label, name, sizeof(np->label));
+    indi_strlcpy(np->label, label[0] ? label : name, sizeof(np->label));
 
-    strncpy(np->format, format, sizeof(np->format));
+    indi_strlcpy(np->format, format, sizeof(np->format));
 
     np->min   = min;
     np->max   = max;
@@ -203,12 +404,9 @@ void IUFillNumber(INumber *np, const char *name, const char *label, const char *
 
 void IUFillText(IText *tp, const char *name, const char *label, const char *initialText)
 {
-    strncpy(tp->name, name, sizeof(tp->name));
+    indi_strlcpy(tp->name, name, sizeof(tp->name));
 
-    if (label[0])
-        strncpy(tp->label, label, sizeof(tp->label));
-    else
-        strncpy(tp->label, name, sizeof(tp->label));
+    indi_strlcpy(tp->label, label[0] ? label : name, sizeof(tp->label));
 
     if (tp->text && tp->text[0])
         free(tp->text);
@@ -226,14 +424,11 @@ void IUFillBLOB(IBLOB *bp, const char *name, const char *label, const char *form
 {
     memset(bp, 0, sizeof(IBLOB));
 
-    strncpy(bp->name, name, sizeof(bp->name));
+    indi_strlcpy(bp->name, name, sizeof(bp->name));
 
-    if (label[0])
-        strncpy(bp->label, label, sizeof(bp->label));
-    else
-        strncpy(bp->label, name, sizeof(bp->label));
+    indi_strlcpy(bp->label, label[0] ? label : name, sizeof(bp->label));
 
-    strncpy(bp->format, format, sizeof(bp->format));
+    indi_strlcpy(bp->format, format, sizeof(bp->format));
 
     bp->blob    = 0;
     bp->bloblen = 0;
@@ -247,16 +442,13 @@ void IUFillBLOB(IBLOB *bp, const char *name, const char *label, const char *form
 void IUFillSwitchVector(ISwitchVectorProperty *svp, ISwitch *sp, int nsp, const char *dev, const char *name,
                         const char *label, const char *group, IPerm p, ISRule r, double timeout, IPState s)
 {
-    strncpy(svp->device, dev, sizeof(svp->device));
+    indi_strlcpy(svp->device, dev, sizeof(svp->device));
 
-    strncpy(svp->name, name, sizeof(svp->name));
+    indi_strlcpy(svp->name, name, sizeof(svp->name));
 
-    if (label[0])
-        strncpy(svp->label, label, sizeof(svp->label));
-    else
-        strncpy(svp->label, name, sizeof(svp->label));
+    indi_strlcpy(svp->label, label[0] ? label : name, sizeof(svp->label));
 
-    strncpy(svp->group, group, sizeof(svp->group));
+    indi_strlcpy(svp->group, group, sizeof(svp->group));
     svp->timestamp[0] = '\0';
 
     svp->p       = p;
@@ -270,16 +462,13 @@ void IUFillSwitchVector(ISwitchVectorProperty *svp, ISwitch *sp, int nsp, const
 void IUFillLightVector(ILightVectorProperty *lvp, ILight *lp, int nlp, const char *dev, const char *name,
                        const char *label, const char *group, IPState s)
 {
-    strncpy(lvp->device, dev, sizeof(lvp->device));
+    indi_strlcpy(lvp->device, dev, sizeof(lvp->device));
 
-    strncpy(lvp->name, name, sizeof(lvp->name));
+    indi_strlcpy(lvp->name, name, sizeof(lvp->name));
 
-    if (label[0])
-        strncpy(lvp->label, label, sizeof(lvp->label));
-    else
-        strncpy(lvp->label, name, sizeof(lvp->label));
+    indi_strlcpy(lvp->label, label[0] ? label : name, sizeof(lvp->label));
 
-    strncpy(lvp->group, group, sizeof(lvp->group));
+    indi_strlcpy(lvp->group, group, sizeof(lvp->group));
     lvp->timestamp[0] = '\0';
 
     lvp->s   = s;
@@ -290,16 +479,13 @@ void IUFillLightVector(ILightVectorProperty *lvp, ILight *lp, int nlp, const cha
 void IUFillNumberVector(INumberVectorProperty *nvp, INumber *np, int nnp, const char *dev, const char *name,
                         const char *label, const char *group, IPerm p, double timeout, IPState s)
 {
-    strncpy(nvp->device, dev, sizeof(nvp->device));
+    indi_strlcpy(nvp->device, dev, sizeof(nvp->device));
 
-    strncpy(nvp->name, name, sizeof(nvp->name));
+    indi_strlcpy(nvp->name, name, sizeof(nvp->name));
 
-    if (label[0])
-        strncpy(nvp->label, label, sizeof(nvp->label));
-    else
-        strncpy(nvp->label, name, sizeof(nvp->label));
+    indi_strlcpy(nvp->label, label[0] ? label : name, sizeof(nvp->label));
 
-    strncpy(nvp->group, group, sizeof(nvp->group));
+    indi_strlcpy(nvp->group, group, sizeof(nvp->group));
     nvp->timestamp[0] = '\0';
 
     nvp->p       = p;
@@ -312,16 +498,13 @@ void IUFillNumberVector(INumberVectorProperty *nvp, INumber *np, int nnp, const
 void IUFillTextVector(ITextVectorProperty *tvp, IText *tp, int ntp, const char *dev, const char *name,
                       const char *label, const char *group, IPerm p, double timeout, IPState s)
 {
-    strncpy(tvp->device, dev, sizeof(tvp->device));
+    indi_strlcpy(tvp->device, dev, sizeof(tvp->device));
 
-    strncpy(tvp->name, name, sizeof(tvp->name));
+    indi_strlcpy(tvp->name, name, sizeof(tvp->name));
 
-    if (label[0])
-        strncpy(tvp->label, label, sizeof(tvp->label));
-    else
-        strncpy(tvp->label, name, sizeof(tvp->label));
+    indi_strlcpy(tvp->label, label[0] ? label : name, sizeof(tvp->label));
 
-    strncpy(tvp->group, group, sizeof(tvp->group));
+    indi_strlcpy(tvp->group, group, sizeof(tvp->group));
     tvp->timestamp[0] = '\0';
 
     tvp->p       = p;
@@ -335,16 +518,13 @@ void IUFillBLOBVector(IBLOBVectorProperty *bvp, IBLOB *bp, int nbp, const char *
                       const char *label, const char *group, IPerm p, double timeout, IPState s)
 {
     memset(bvp, 0, sizeof(IBLOBVectorProperty));
-    strncpy(bvp->device, dev, sizeof(bvp->device));
+    indi_strlcpy(bvp->device, dev, sizeof(bvp->device));
 
-    strncpy(bvp->name, name, sizeof(bvp->name));
+    indi_strlcpy(bvp->name, name, sizeof(bvp->name));
 
-    if (label[0])
-        strncpy(bvp->label, label, sizeof(bvp->label));
-    else
-        strncpy(bvp->label, name, sizeof(bvp->label));
+    indi_strlcpy(bvp->label, label[0] ? label : name, sizeof(bvp->label));
 
-    strncpy(bvp->group, group, sizeof(bvp->group));
+    indi_strlcpy(bvp->group, group, sizeof(bvp->group));
     bvp->timestamp[0] = '\0';
 
     bvp->p       = p;
@@ -555,13 +735,12 @@ int IUSnoopBLOB(XMLEle *root, IBLOBVectorProperty *bvp)
 
             XMLAtt *fa = findXMLAtt(ep, "format");
             XMLAtt *sa = findXMLAtt(ep, "size");
-            XMLAtt *ec = findXMLAtt(ep, "enclen");
-            if (fa && sa && ec)
+            if (fa && sa)
             {
-                int enclen  = atoi(valuXMLAtt(ec));
-                assert_mem(bp->blob = realloc(bp->blob, 3 * enclen / 4));
-                bp->bloblen = from64tobits_fast(bp->blob, pcdataXMLEle(ep), enclen);
-                strncpy(bp->format, valuXMLAtt(fa), MAXINDIFORMAT);
+                int base64datalen = pcdatalenXMLEle(ep);
+                assert_mem(bp->blob = realloc(bp->blob, 3 * base64datalen / 4));
+                bp->bloblen = from64tobits_fast(bp->blob, pcdataXMLEle(ep), base64datalen);
+                indi_strlcpy(bp->format, valuXMLAtt(fa), MAXINDIFORMAT);
                 bp->size = atoi(valuXMLAtt(sa));
             }
         }
